<?php declare(strict_types=1);

namespace Drupal\commerce_product_feeds\Normalizer\Cache;

use Drupal\commerce_product_feeds\Normalizer\Value\CacheableNormalization;
use Drupal\commerce_product_feeds\Normalizer\Value\CacheableOmission;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Service to wrap gets and sets to the normalized value cache.
 */
class NormalizedValueCacher implements NormalizedValueCacherInterface {

  /**
   * Cache bin.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBin;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBin
   *   Cache backend.
   */
  public function __construct(CacheBackendInterface $cacheBin) {
    $this->cacheBin = $cacheBin;
  }

  /**
   * {@inheritDoc}
   */
  public function get(ContentEntityInterface $entity): ?CacheableNormalization {
    $cached = $this->cacheBin->get($this->getCid($entity));
    return $cached
      ? new CacheableNormalization((new CacheableMetadata())->setCacheTags($cached->tags), $cached->data)
      : NULL;
  }

  /**
   * {@inheritDoc}
   */
  public function set(CacheableNormalization $object, ContentEntityInterface $entity): void {
    assert(
      ($object instanceof CacheableOmission) || (count(array_diff($object->getCacheContexts(), ['url.query_args:v'])) === 0),
      new \RuntimeException('Cache contexts for normalization to product feeds not [yet] supported.')
    );
    $this->cacheBin->set(
      $this->getCid($entity),
      ($object instanceof CacheableOmission) ? [] : $object->getNormalization(),
      $object->getCacheMaxAge(),
      $entity->getCacheTags()
    );
  }

  /**
   * Get a cache ID for an object.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $object
   *   Entity to retrieve cache ID for.
   *
   * @return string
   */
  protected function getCid(ContentEntityInterface $object): string {
    return $object->getEntityTypeId() . ':' . $object->id();
  }

  /**
   * @inheritDoc
   */
  public function clearCache(): void {
    $this->cacheBin->deleteAll();
  }

}
